@@ -1,8 +1,8 @@
-from behave_cli.expect.steps import *
-from behave_cli.files.steps import *
-from behave_cli.api.steps import *
-from behave_cli.s3mock.steps import *
-from behave_cli.netrc.steps import *
+from behave_cli.expect.steps import *  # noqa
+from behave_cli.files.steps import *  # noqa
+from behave_cli.api.steps import *  # noqa
+from behave_cli.s3mock.steps import *  # noqa
+from behave_cli.netrc.steps import *  # noqa
 from behave_cli.s3mock import setup as s3setup
 from behave_cli.s3mock import teardown as s3teardown
 from behave_cli.s3mock.steps import s3bucket_named
@@ -20,3 +20,7 @@ def mp_setup(ctx):
 
 def mp_teardown(ctx):
     s3teardown(ctx)
+
+
+def cert_vars(ctx):
+    return {'certid': ctx.certid}
