@@ -1,16 +1,15 @@
 from __future__ import unicode_literals
-import platform
 
 try:
     from lacli.version import __version__
 except ImportError:
-    __version__ = "0.2.3"
+    __version__ = "0.2.6.1"
+
 
 def get_client_info(terse=False):
+    from lacore import get_client_info as lacore_info
     el = ["Longaccess client"]
     el.append(__version__)
-    el.append(platform.platform(True, terse))
-    el.append("py"+platform.python_version())
-    el.append(platform.machine())
+    el.append(lacore_info(terse))
     return " ".join(el)
 # vim: et:sw=4:ts=4
