@@ -5,29 +5,37 @@ import pyaml
 import sys
 import errno
 import operator
+
 from pipes import quote
 from lacli.log import getLogger
 from lacli.upload import Upload, UploadState
+from lacli.capsule import archive_capsule
 from lacli.archive import restore_archive
-from lacli.adf import archive_size, Certificate, Archive, Meta, creation
-from lacli.decorators import command, login, block
-from lacli.exceptions import DecryptionError, PauseEvent
+from lacore.adf.util import archive_size, creation
+from lacore.adf.elements import Archive, Certificate, Meta, Signature
+from lacore.async import block
+from lacli.cmdutil import command
+from lacli.exceptions import PauseEvent
 from lacli.compose import compose
 from lacli.progress import ConsoleProgressHandler
 from lacli.server.interface.ClientInterface.ttypes import ArchiveStatus
 from lacli.basecmd import LaBaseCommand
+from lacli.loginutil import login
+from lacli.certinput import ask_key
 from twisted.internet import defer, reactor, task
 
 from richtext import RichTextUI as UIClass
+from richtext import format_size
 ui = UIClass()
 
+
 class LaCertsCommand(LaBaseCommand):
     """Manage Long Access Certificates
 
     Usage: lacli certificate list
            lacli certificate print <cert_id>
            lacli certificate export <cert_id>
-           lacli certificate import <filename>
+           lacli certificate import [<filename>]
            lacli certificate delete <cert_id> [<srm>]
            lacli certificate --help
 
@@ -51,7 +59,8 @@ class LaCertsCommand(LaBaseCommand):
             line.append(options["<cert_id>"])
         elif options['import']:
             line.append("import")
-            line.append(options["<filename>"])
+            if options['<filename>']:
+                line.append(options["<filename>"])
         return " ".join(line)
 
     @command()
@@ -83,7 +92,7 @@ class LaCertsCommand(LaBaseCommand):
                 return
             print "Deleting certificate", cert_id, "in",
             for c in ui.countdown(5):
-                yield 
+                yield
             print "... deleting"
 
         fname = self.cache.shred_cert(cert_id, _countdown(), srm)
@@ -126,8 +135,25 @@ class LaCertsCommand(LaBaseCommand):
     @command(filename=unicode)
     def do_import(self, filename=None):
         """
-        Usage: import <filename>
+        Usage: import [<filename>]
         """
+        if filename is None:
+            cert = self.input("Enter the certificate ID:")
+            if cert in self.cache.certs():
+                print "A certificate with this ID already exists"
+            else:
+                title = self.input("Enter the certificate title:")
+                key = ask_key()
+                if key is not None:
+                    print "Imported certificate {}".format(
+                        self.cache.save_cert({
+                            'archive': Archive(title,
+                                               Meta('zip', 'aes-256-ctr')),
+                            'cert': Certificate(key),
+                            'signature': Signature(aid=cert, uri='',)
+                        })[0])
+            return
+
         certs = self.cache.certs([filename])
         if len(certs) > 0:
             for key, cert in certs.iteritems():
@@ -144,7 +170,7 @@ class LaCapsuleCommand(LaBaseCommand):
     """Manage Long Access Capsules
 
     Usage: lacli capsule list
-           lacli capsule create <title>
+           lacli capsule archives [<capsule>]
            lacli capsule --help
 
     """
@@ -154,6 +180,10 @@ class LaCapsuleCommand(LaBaseCommand):
         line = []
         if options['list']:
             line.append("list")
+        if options['archives']:
+            line.append('archives')
+            if options['<capsule>']:
+                line.append(options['<capsule>'])
         return " ".join(line)
 
     @login
@@ -169,25 +199,62 @@ class LaCapsuleCommand(LaBaseCommand):
                 ui.print_capsules_header()
                 for capsule in capsules:
                     n = n+1
-                    ui.print_capsules_line(capsule = {
+                    ui.print_capsules_line(capsule={
                         'num': n,
                         'size': capsule['size'],
                         'remaining': capsule['remaining'],
                         'title': capsule['title'],
                         'id': capsule['id'],
                         'created': capsule['created'],
-                        'expires':capsule['expires'],
+                        'expires': capsule['expires'],
                     })
             else:
                 print "No available capsules."
         except Exception as e:
             print "error: " + str(e)
 
+    @login
+    @command(capsule=int)
+    def do_archives(self, capsule=None):
+        """
+        Usage: archives [<capsule>]
+        """
+        try:
+            if capsule is not None:
+                capsules = self.session.capsule_ids()
+                if capsule not in capsules:
+                    print "No such capsule"
+                    return
+                capsule = capsules[capsule]
+            archives = self.session.archives()
+            if capsule is not None:
+                archives = [a for a in archives
+                            if a['capsule'] == capsule['resource_uri']]
+            if len(archives) > 0:
+                capsules = dict(
+                    [(c['resource_uri'], c['title'])
+                     for c in self.session.capsules()])
+                ui.print_archives_header()
+                for n, archive in enumerate(archives):
+                    ui.print_archives_line(archive={
+                        'num': n+1,
+                        'size': format_size(int(archive['size'])),
+                        'title': archive['title'],
+                        'cert': archive['key'],
+                        'created': archive['created'],
+                        'status': "COMPLETE",
+                        'capsule': capsules.get(archive['capsule'], '')
+                    })
+            else:
+                print "No available archives."
+        except Exception as e:
+            print "error: " + str(e)
+
 
 class LaArchiveCommand(LaBaseCommand):
     """Upload a file to Long Access
 
-    Usage: lacli archive upload [-n <np>] [<index>] [<capsule>]
+    Usage: lacli archive upload [-T] [-n <np>] [<index>] [<capsule>]
            lacli archive list
            lacli archive status <index>
            lacli archive create <dirname> -t <title> [--desc <description>]
@@ -197,6 +264,7 @@ class LaArchiveCommand(LaBaseCommand):
            lacli archive --help
 
     Options:
+        -T, --test                          upload to sandbox
         -n <np>, --procs <np>               number of processes [default: auto]
         -t <title>, --title <title>         title for prepared archive
         --desc <description>                description for prepared archive
@@ -214,6 +282,7 @@ class LaArchiveCommand(LaBaseCommand):
         super(LaArchiveCommand, self).__init__(*args, **kwargs)
         UploadState.init(self.cache)
         self.nprocs = None
+        self.sandbox = False
 
     def setopt(self, options):
         try:
@@ -222,6 +291,8 @@ class LaArchiveCommand(LaBaseCommand):
         except ValueError:
             print "error: illegal value for 'procs' parameter."
             raise
+        if options['--test']:
+            self.sandbox = True
 
     def makecmd(self, options):
         self.setopt(options)
@@ -285,33 +356,28 @@ class LaArchiveCommand(LaBaseCommand):
             docs = docs[index-1][1]
             size = docs['archive'].meta.size
 
-            capsules = self.session.capsule_ids()
+            capsules = self.session.capsule_ids(
+                size if self.safe else None)
             if capsule is not None:
                 capsule = capsules.get(capsule)
-                if not capsule:
-                    _error += "no such capsule"
-                elif capsule.get('remaining', 0) < size:
-                    _error += "archive too big for capsule"
-                    if self.safe is True:
-                        capsule = None
             elif len(capsules) > 0:
-                for i, c in capsules.iteritems():
-                    if c.get('remaining', 0) > size:
-                        capsule = c
-                if not capsule:
-                    _error += "no capsules with available space found."
-            else:
+                capsule = capsules.itervalues().next()
+
+            if capsule is None:
                 _error += "no capsules found"
 
-        if fname and capsule:
+        if 'links' not in docs:
+            _error += "no local copy exists."
+            link = False
+        else:
             link = docs['links']
-
+        if link and fname and capsule:
             if link.upload or link.download:
                 print "upload is already completed"
             else:
                 try:
-                    size = docs['archive'].meta.size
-                    with UploadState.get(fname, size, capsule) as state:
+                    with UploadState.get(
+                            fname, size, capsule, self.sandbox) as state:
                         handler = ConsoleProgressHandler(
                             maxval=size, fname=fname, state=state)
                         with handler as progq:
@@ -321,6 +387,10 @@ class LaArchiveCommand(LaBaseCommand):
                             print "Upload finished, waiting for verification"
                             print "Press Ctrl-C to check manually later"
                             status = self._poll_status(saved['link'])
+                            while status is None:
+                                getLogger().debug("couldn't get status, "
+                                                  "retrying..")
+                                status = self._poll_status(saved['link'])
                             if status['status'] == "error":
                                 print "status: error"
                                 self.cache.upload_error(fname)
@@ -348,7 +418,8 @@ class LaArchiveCommand(LaBaseCommand):
                                         else:
                                             raise e
                             else:
-                                print "Unknown status received:", status['status']
+                                st = status['status']
+                                print "Unknown status received:", st
 
                     print "\ndone."
                 except PauseEvent:
@@ -367,7 +438,7 @@ class LaArchiveCommand(LaBaseCommand):
         if u['status'] == "completed" or u['status'] == "error":
             defer.returnValue(u)
         else:
-            yield task.deferLater(reactor, 5.0, _poll_status_async, link)
+            yield task.deferLater(reactor, 5.0, self._poll_status_async, link)
 
     @block
     def _poll_status(self, link):
@@ -378,7 +449,7 @@ class LaArchiveCommand(LaBaseCommand):
         archive = docs['archive']
         auth = docs['auth']
         path = self.cache.data_file(docs['links'])
-        op = yield self.session.upload(state.capsule, archive, state)
+        op = yield self.session.upload(archive, state)
         status = yield op.status  # init uri and status
         if status['status'] == 'pending':
             if state.uri is None:
@@ -389,12 +460,14 @@ class LaArchiveCommand(LaBaseCommand):
             status = yield op.finalize(auth, state.keys)
         if status['status'] != 'failed':
             account = yield self.session.async_account
-            saved = yield self.cache.save_upload(fname, docs, op.uri, account)
+            saved = yield self.cache.save_upload(
+                fname, docs, op.uri, account, state.capsule['title'])
             defer.returnValue(saved)
 
     def upload(self, docs, fname, progq, state):
         try:
-            state.deferred_upload = self.upload_async(docs, fname, progq, state)
+            state.deferred_upload = self.upload_async(
+                docs, fname, progq, state)
             return state.wait_for_upload()
         except KeyboardInterrupt:
             getLogger().debug("interrupted.", exc_info=True)
@@ -445,7 +518,8 @@ class LaArchiveCommand(LaBaseCommand):
 
         if len(archives):
             ui.print_archives_header()
-            bydate = sorted(archives, key=compose(creation, operator.itemgetter(1)))
+            bydate = sorted(archives,
+                            key=compose(creation, operator.itemgetter(1)))
             for n, docs in enumerate(bydate):
                 fname = docs[0]
                 archive = docs[1]
@@ -470,14 +544,16 @@ class LaArchiveCommand(LaBaseCommand):
                     status = "UNKNOWN"
                 title = archive['archive'].title
                 size = archive_size(archive['archive'])
-                ui.print_archives_line(archive = {
-                        'num': n+1,
-                        'size': size,
-                        'title': title,
-                        'status': status,
-                        'cert':cert,
-                        'created':archive['archive'].meta.created
-                    })
+
+                ui.print_archives_line(archive={
+                    'num': n+1,
+                    'size': size,
+                    'title': title,
+                    'status': status,
+                    'cert': cert,
+                    'created': archive['archive'].meta.created,
+                    'capsule': archive_capsule(archive) or '-'
+                })
                 if self.debug > 2:
                     for doc in archive.itervalues():
                         pyaml.dump(doc, sys.stdout)
@@ -509,12 +585,22 @@ class LaArchiveCommand(LaBaseCommand):
                     url = upload['links'].upload
                     status = self.session.upload_status(url)
                     if status['status'] == "completed":
+                        print "status: complete"
                         cert, f = self.cache.save_cert(
                             self.cache.upload_complete(fname, status))
                         if f:
                             print "Certificate", cert, "saved:", f
                         else:
                             print "Certificate", cert, "already exists.\n"
+                        for i in range(3):
+                            try:
+                                UploadState.reset(fname)
+                                break
+                            except OSError as e:
+                                if i < 3 and e.errno == errno.EACCES:
+                                    continue
+                                else:
+                                    raise e
                         print " ".join(("Use lacli certificate list",
                                         "to see your certificates, or",
                                         "lacli certificate --help for",
@@ -573,11 +659,13 @@ class LaArchiveCommand(LaBaseCommand):
                                         'tmp', write=True), _print)
                     print "archive restored."
                 if cert_id in certs:
-                    print "Decrypting archive. This may take some time depending on the size of your archive."
+                    print "Decrypting archive. This may take some time",
+                    print "depending on the size of your archive."
                     extract(certs[cert_id]['cert'], certs[cert_id]['archive'])
                 else:
-                    getLogger().debug("Key input gui unavailable and key not found",
-                                          exc_info=True)
+                    getLogger().debug(
+                        "Key input gui unavailable and key not found",
+                        exc_info=True)
                     print "error: key not found"
 
             except Exception as e:
@@ -600,7 +688,7 @@ class LaArchiveCommand(LaBaseCommand):
             print "No such archive."
         else:
             fname = docs[index-1][0]
-            if fname in UploadState.states:
+            if UploadState.has_state(fname):
                 UploadState.reset(fname)
                 print "archive upload status reset."
             else:
@@ -660,4 +748,53 @@ class LaArchiveCommand(LaBaseCommand):
                 print "Please remove manually"
             else:
                 print "Deleted archive", archive.title
+
+
+class LaFetchCommand(LaBaseCommand):
+    """Fetch Long Access Certificates
+
+    Usage: lacli fetch <archiveid> [<key>]
+    """
+    prompt = 'lacli:fetch> '
+
+    def makecmd(self, options):
+        line = ["fetch"]
+        line.append(options['<archiveid>'])
+        if options['<key>']:
+            line.append(options['<key>'])
+        return " ".join(line)
+
+    @login
+    @command(archiveid=str, key=str)
+    def do_fetch(self, archiveid, key=None):
+        """
+        Usage: fetch <archiveid> [<key>]
+        """
+        try:
+            archives = [a for a in self.session.archives()
+                        if a['key'] == archiveid]
+            if len(archives) == 0:
+                print "Archive not found"
+            elif len(archives) > 1:
+                print "More than one archive with same ID!"
+            else:
+                archive = archives[0]
+                print "title:", archive['title']
+                print "description:", archive['description']
+                print "expiration:", archive['expires'].strftime("%Y-%m-%d")
+                print "created:", archive['created'].strftime("%Y-%m-%d")
+                key = ask_key() if key is None else key.decode('hex')
+                print "Fetched certificate {}".format(
+                    self.cache.save_cert({
+                        'archive': Archive(archive['title'],
+                                           Meta('zip', 'aes-256-ctr'),
+                                           description=archive['description']),
+                        'cert': Certificate(key),
+                        'signature': Signature(
+                            aid=archive['key'], uri='',
+                            created=archive['created'],
+                            expires=archive['expires'])
+                    })[0])
+        except Exception as e:
+            print "error: " + str(e)
 # vim: et:sw=4:ts=4
