@@ -1,18 +1,15 @@
 #!/bin/sh
+python setup.py clean
 echo "Cleaning .pyc files"
 find . -type f -name "*.pyc" -print0 | xargs -r -0 rm
 echo "Running PEP8 checks"
-pep8 .
+flake8 --max-complexity 23 --exclude=ClientInterface
 if [ $? -ne 0 ]
 then
-    echo "PEP8 violations detected. Stopping run."
+    echo "pyflake8 errors detected. Stopping run."
     exit 1
 fi
 
-./node_modules/.bin/robohydra mockapi.conf &
-robohydra_pid=$!
-export MOCK_API_URL='http://localhost:3000/'
-
 nosetests
 if [ $? -ne 0 ]
 then
@@ -21,6 +18,11 @@ then
 fi
 echo "nosetests passed"
 
+
+./node_modules/.bin/robohydra mockapi.conf &
+robohydra_pid=$!
+export MOCK_API_URL='http://localhost:3000/'
+
 behave -t ~@dev
 
 kill $robohydra_pid
